import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from scipy.cluster.hierarchy import linkage
from scipy.cluster.hierarchy import dendrogram

#   ExcelファイルをPandasのDataFrame型として読み込む
xlfile = pd.ExcelFile('CATAData.xlsx')
df = pd.read_excel(xlfile)

#    １行目の値（米の種類）
col_names = df.keys()         #    Excelの１行目
col_names = col_names[1:]     #    ２番目以降を取り出す

#    '評価用語'の列を取り出す
row_names = df['評価用語']

#    セル内の数値
df_data = df.values           #   ２行目以降が取り出される
df_data = df_data[:,1:]       #    取り出した各行の２列目以降を取り出す

#    セル内の数値を実数型に変換
data = np.asfarray(df_data)

#     dataの特異値分解SVD
u, s, vh = np.linalg.svd(data, full_matrices = False)

print('\ns =', s)

#     特異値の平方根による重み付け
col_coord = np.diag(s**0.5) @ vh
row_coord = u @ np.diag(s**0.5)

#     日本語表示用
plt.rcParams['font.family'] = ['Yu Gothic',
'Noto Sans CJK JP','Hiragino Maru Gothic Pro']

#
#     重み付けられた特異ベクトルによる描画
#
plt.plot(row_coord.T[0],row_coord.T[1],'^',c = 'k')
plt.plot([], '^',c = 'k',label = '評価項目')
plt.plot(col_coord[0],col_coord[1],'v',c = 'k')
plt.plot([], 'v', c = 'k', label = '米')
for i in range(len(col_names)):
    plt.text(col_coord[0][i], col_coord[1][i],
             col_names[i]) 
for i in range(len(row_names)):
    plt.text(row_coord.T[0][i], row_coord.T[1][i],
             row_names[i],rotation = -20,
             rotation_mode = 'anchor')
plt.legend()
plt.savefig('FigMap.png')
plt.show()

#
#       クラスター分析のためにクロス表の行と列の座標を統合
#
mrg_coord = np.concatenate((row_coord, col_coord.T), axis = 0)
mrg_label = np.concatenate((row_names, col_names), axis = 0)
#
#       クラスター分析
#
row_clusters = linkage(mrg_coord, method = 'ward')
fig = plt.figure(figsize = (15, 8), facecolor = 'white')
row_dendr = dendrogram(row_clusters, labels = mrg_label,
                       orientation = 'right')
plt.savefig('FigCluster.png')
plt.show()


